from argparse import ArgumentParser
from getpass import getpass
from functools import reduce
import sys

from vernum.command import VerNumCommand
from vernum.scheme import Scheme


class MaxCommand(VerNumCommand):
    """Return the highest of a set of version strings"""

    name = 'max'

    @VerNumCommand.wrap
    def execute(self):
        """Return the highest of a set of version numbers"""
        verstringlist = self.input.text.split('\n')
        vernums = [v for s in verstringlist if (v := self.scheme.parse(s))]
        if vernums:
            maxval = max(vernums, key=lambda v: v.val)
            self.status = f"Found max version number {maxval}"
            return str(maxval)
        else:
            self.status = f"Version list is empty"

        # current = scheme.parse(self.input)
        # next = getattr(current.increment, self.increment)
        # self.status = f'Version number incremented from {current} to {next}'
        # return str(next)

        # zeroval = (0, 0, 0, -1, -1)
        # verstringlist = self.input.split('\n')
        # vals = [v for s in verstringlist if (v := string2val(s))]
        # maxval = reduce(max, vals, zeroval)
        # self.status = 'Done'
        # return val2string(*maxval)



from dataclasses import dataclass
from argparse import ArgumentParser
import os

from wizlib.command import WizCommand
from wizlib.input_handler import InputHandler
from wizlib.config_handler import ConfigHandler

from vernum.scheme import Scheme
from vernum.error import VerNumError


class VerNumCommand(WizCommand):

    default = 'full'
    handlers = [InputHandler, ConfigHandler]

    @property
    def scheme(self):
        name = self.config.get('vernum-scheme') or 'patch'
        scheme = Scheme.family_member('name', name)
        if not scheme:
            raise VerNumError(f"Invalid scheme '{name}'")
        return scheme



from dataclasses import dataclass
import re

from vernum.scheme import Scheme


class MinorAlphaBetaScheme(Scheme):
    """
    Supports alpha and beta prepatch versions, applied at the minor level.

    Example: 3.4.alpha2
    """

    name = 'minor-alpha-beta'

    FORMAT = re.compile(
        r"^v?(\d+)\.(\d+)\.(?:(\d+)|beta(\d+)|alpha(\d+))$")
    INCREMENTS = ['major', 'minor', 'patch', 'beta', 'alpha']

    def __str__(self):
        if self.val:
            r = f"{str(self.val[0])}.{str(self.val[1])}."
            if self.val[4] > -1:
                return r + f"alpha{str(self.val[4])}"
            if self.val[3] > -1:
                return r + f"beta{str(self.val[3])}"
            return r + str(self.val[2])

    def increment(self, up: str):
        major, minor, patch, beta, alpha = self.val
        if up == 'alpha':
            if alpha + 1:
                alpha += 1
            elif beta + 1:
                beta += 1
            else:
                return
        elif up == 'beta':
            if beta + 1:
                beta += 1
            elif alpha + 1:
                beta = 1
                alpha = -1
            else:
                return
        elif up == 'patch':
            if patch + 1:
                patch += 1
            elif beta + 1 or alpha + 1:
                patch = 0
                beta = alpha = -1
            else:
                return
        elif up == 'minor':
            minor += 1
            alpha = 1
            patch = beta = -1
        elif up == 'minor-zero':
            minor += 1
            patch = 0
            beta = alpha = -1
        elif up == 'major':
            major += 1
            minor = 0
            alpha = 1
            patch = beta = -1
        elif up == 'major-zero':
            major += 1
            minor = patch = 0
            beta = alpha = -1
        return MinorAlphaBetaScheme(major, minor, patch, beta, alpha)
